
/* SQL script to define new international objects to a database */

/* Note: there are three locations in this script to modify the database name */


create database '/tmp/example.gdb' user 'SYSDBA' password 'masterkey';
show version;

input cs_util.sql;

execute procedure create_character_set ('EXAMPLE_CHARSET', 251, 1);
execute procedure create_character_set_alias
        ('EXAMPLE_BY_ANOTHER_NAME', 'EXAMPLE_CHARSET');
execute procedure create_collation ('EXAMPLE_COLLATION', 250, 'WIN1252');


Commit;

select * from show_character_sets;
select * from show_collations('WIN1252');

commit;

/* reconnect to the database, using the new charset as attachment default */
set names EXAMPLE_CHARSET;
connect '/tmp/example.gdb' user 'SYSDBA' password 'masterkey';

/* Create some tables using the character sets & collations */

create table x1 (a character(25) character set EXAMPLE_CHARSET);
create table x2 (a character(25) character set win1252
                                 collate EXAMPLE_COLLATION);
create table x3 (a character(25) character set EXAMPLE_CHARSET
                                 collate EXAMPLE_CHARSET);
commit;
show table x1;
show table x2;
show table x3;


/* In the simple collation example, simple words 'The', 'A', & 'An' are ignored */

insert into x2 (a) values ('The Wind and the lion');
insert into x2 (a) values ('Wind and the lion');
insert into x2 (a) values ('A Wind and the lion');
insert into x2 (a) values ('An Wind and the lion');
insert into x2 (a) values ('X');
insert into x2 (a) values ('V');
commit;

set echo on;
select * from x2 order by a;
select * from x2 order by a descending;
create index x2 on x2(a);
select * from x2 order by a;
set echo off;

insert into x3 (a) values ('a');
insert into x3 (a) values ('b');
insert into x3 (a) values ('c');
insert into x3 (a) values ('z');
insert into x3 (a) values ('A');
insert into x3 (a) values ('B');
insert into x3 (a) values ('C');
insert into x3 (a) values ('Z');
commit;

set echo on;
select * from x3;
select * from x3 order by a;
select * from x3 order by a descending;
create index x3 on x3(a);
select * from x3 order by a;
commit;

/* Now set the DEFAULT CHARACTER SET of the database to the new character set
   NOTE: We cannot do this when the database is created as the charset
         must be declared to the database first */

execute procedure set_default_character_set ('EXAMPLE_CHARSET');
commit;

/* Reload the database so the metadata is reloaded */
connect '/tmp/example.gdb' user 'SYSDBA' password 'masterkey';

show database;

create table x4 (a character(25));

commit;
show table x1;
show table x2;
show table x3;
show table x4;


commit;

/* These drops will fail as objects are in use */
execute procedure drop_collation ('EXAMPLE_COLLATION');
execute procedure drop_collation ('EXAMPLE_CHARSET');
execute procedure drop_character_set ('EXAMPLE_CHARSET');
commit;


drop table x2;
execute procedure drop_collation ('EXAMPLE_COLLATION');
execute procedure drop_character_set_alias ('EXAMPLE_BY_ANOTHER_NAME');

drop table x1;
drop table x3;
drop table x4;
execute procedure drop_character_set ('EXAMPLE_CHARSET');
commit;


select * from show_character_sets;
select * from show_collations ('WIN1252');

/* drop database; */
quit;
