
/* cs_util.sql
 * SQL script defining procedures to manage International objects
 */

/*
connect 'c:\database\charset.gdb' user sysdba password 'masterkey';

drop procedure create_character_set;
drop procedure create_character_set_alias;
drop procedure create_collation;
drop procedure set_default_character_set;
drop procedure drop_character_set;
drop procedure drop_collation;
drop procedure drop_character_set_alias;
drop procedure show_collations;
drop procedure show_character_sets;
drop exception intl_obj_in_use;
commit;
*/
 
SET TERM ;; ;

CREATE PROCEDURE CREATE_CHARACTER_SET_ALIAS (
        new_alias_name          CHARACTER(31) CHARACTER SET UNICODE_FSS,
        existing_name           CHARACTER(31) CHARACTER SET UNICODE_FSS)
AS
BEGIN
INSERT INTO rdb$types (
    rdb$field_name,
    rdb$type,
    rdb$type_name)
values
    ('RDB$CHARACTER_SET_NAME',
     (SELECT RDB$CHARACTER_SET_ID FROM RDB$CHARACTER_SETS WHERE
        RDB$CHARACTER_SET_NAME = :existing_name),
     :new_alias_name);
END;;

CREATE PROCEDURE CREATE_COLLATION (
        new_collation_name      CHARACTER(31) CHARACTER SET UNICODE_FSS,
        new_collation_id        SMALLINT,
        character_set_name      CHARACTER(31) CHARACTER SET UNICODE_FSS)
AS
BEGIN
INSERT INTO rdb$collations (
    rdb$collation_name,
    rdb$collation_id,
    rdb$character_set_id
    )
VALUES
    (
    :new_collation_name,
    :new_collation_id,
     (SELECT RDB$CHARACTER_SET_ID FROM RDB$CHARACTER_SETS WHERE
        RDB$CHARACTER_SET_NAME = :character_set_name)
    );
END;;


CREATE PROCEDURE CREATE_CHARACTER_SET (
        new_name                CHARACTER(31) CHARACTER SET UNICODE_FSS,
        new_id                  SMALLINT,
        new_bytes_per_character SMALLINT)
AS
BEGIN
INSERT INTO rdb$character_sets (
        RDB$CHARACTER_SET_NAME,
        RDB$DEFAULT_COLLATE_NAME,
        RDB$CHARACTER_SET_ID,
        RDB$SYSTEM_FLAG,
        RDB$BYTES_PER_CHARACTER)
VALUES (
        :new_name,
        :new_name,
        :new_id,
        0,
        :new_bytes_per_character);
EXECUTE PROCEDURE CREATE_CHARACTER_SET_ALIAS (:new_name, :new_name);
EXECUTE PROCEDURE CREATE_COLLATION (:new_name, 0, :new_name);
END;;


CREATE PROCEDURE SET_DEFAULT_CHARACTER_SET (
        new_default             CHARACTER(31) CHARACTER SET UNICODE_FSS)
AS
BEGIN
UPDATE RDB$DATABASE SET RDB$CHARACTER_SET_NAME = :new_default;
END;;

CREATE EXCEPTION INTL_OBJ_IN_USE       'The specified object is in use';;

CREATE PROCEDURE DROP_CHARACTER_SET (
        charset_name            CHARACTER(31) CHARACTER SET UNICODE_FSS
)
AS
DECLARE VARIABLE   charset_id      SMALLINT;
BEGIN
/* Check to see if charset is in use */
charset_id = NULL;
SELECT RDB$CHARACTER_SET_ID FROM RDB$CHARACTER_SETS WHERE
        RDB$CHARACTER_SET_NAME = :charset_name
        INTO :charset_id;

IF (EXISTS (SELECT RDB$CHARACTER_SET_ID FROM RDB$FIELDS
        WHERE RDB$CHARACTER_SET_ID = :charset_id)) THEN
        EXCEPTION intl_obj_in_use;

IF (EXISTS (SELECT RDB$CHARACTER_SET_ID FROM RDB$FUNCTION_ARGUMENTS
        WHERE RDB$CHARACTER_SET_ID = :charset_id)) THEN
        EXCEPTION intl_obj_in_use;

DELETE FROM RDB$CHARACTER_SETS WHERE RDB$CHARACTER_SET_NAME = :charset_name;
DELETE FROM RDB$COLLATIONS WHERE RDB$CHARACTER_SET_ID = :charset_id;
DELETE FROM RDB$TYPES WHERE RDB$TYPE_NAME = :charset_name AND
        RDB$FIELD_NAME = 'RDB$CHARACTER_SET_NAME' AND
        RDB$TYPE = :charset_id;
UPDATE RDB$DATABASE SET RDB$CHARACTER_SET_NAME = NULL
        WHERE RDB$CHARACTER_SET_NAME = :charset_name;
END;;



CREATE PROCEDURE DROP_CHARACTER_SET_ALIAS (
        charset_name            CHARACTER(31) CHARACTER SET UNICODE_FSS
)
AS
BEGIN
DELETE FROM RDB$TYPES WHERE RDB$TYPE_NAME = :charset_name AND
        RDB$FIELD_NAME = 'RDB$CHARACTER_SET_NAME';
END;;



CREATE PROCEDURE DROP_COLLATION (
        collation_name            CHARACTER(31) CHARACTER SET UNICODE_FSS
)
AS
DECLARE VARIABLE charset_id     SMALLINT;
DECLARE VARIABLE collate_id     SMALLINT;
BEGIN
/* Check to see if collation is in use */

charset_id = NULL;
collate_id = NULL;
SELECT RDB$CHARACTER_SET_ID, RDB$COLLATION_ID FROM RDB$COLLATIONS WHERE
        RDB$COLLATION_NAME = :collation_name
        INTO :charset_id, :collate_id;

IF (EXISTS (SELECT RDB$COLLATION_ID FROM RDB$FIELDS
        WHERE RDB$CHARACTER_SET_ID = :charset_id AND
              RDB$COLLATION_ID = :collate_id)) THEN
        EXCEPTION intl_obj_in_use;

DELETE FROM RDB$COLLATIONS WHERE RDB$COLLATION_NAME = :collation_name;
END;;


/* Set permissions on the new procedures */

GRANT EXECUTE ON PROCEDURE CREATE_CHARACTER_SET TO SYSDBA;;
GRANT EXECUTE ON PROCEDURE CREATE_COLLATION TO SYSDBA;;
GRANT EXECUTE ON PROCEDURE CREATE_CHARACTER_SET_ALIAS TO SYSDBA;;
GRANT EXECUTE ON PROCEDURE SET_DEFAULT_CHARACTER_SET TO SYSDBA;;
GRANT EXECUTE ON PROCEDURE DROP_CHARACTER_SET TO SYSDBA;;
GRANT EXECUTE ON PROCEDURE DROP_COLLATION TO SYSDBA;;
GRANT EXECUTE ON PROCEDURE DROP_CHARACTER_SET_ALIAS TO SYSDBA;;
GRANT EXECUTE ON PROCEDURE CREATE_COLLATION
        TO PROCEDURE CREATE_CHARACTER_SET;;
GRANT EXECUTE ON PROCEDURE CREATE_CHARACTER_SET_ALIAS
        TO PROCEDURE CREATE_CHARACTER_SET;;


/* Some procedures to show information */

create procedure show_character_sets
returns (character_set character(30) character set ascii,
         character_set_id smallint)
as
begin
for select rdb$character_set_name, rdb$character_set_id
    from rdb$character_sets order by 1
    into :character_set, :character_set_id do
    begin
    suspend;
    end
end;;

create procedure show_collations (character_set character(30))
returns (collation character(30) character set ascii,
        collation_id integer)
as
begin
for select rdb$collation_name, rdb$collation_id
    from rdb$collations join rdb$character_sets on
      rdb$collations.rdb$character_set_id = rdb$character_sets.rdb$character_set_id
    where rdb$character_sets.rdb$character_set_name = :character_set
    order by 1
    into :collation, :collation_id do
    begin
    suspend;
    end
end;;

GRANT EXECUTE ON PROCEDURE SHOW_CHARACTER_SETS TO PUBLIC;;
GRANT EXECUTE ON PROCEDURE SHOW_COLLATIONS TO PUBLIC;;

set term ; ;;
commit;


