
/*
 * intlobj.h
 *
 * Interface between InterBase Engine and language driver modules.
 *
 * copyright (c) 1992 - 1999 by Inprise Corporation
 */


#ifndef _JRD_INTLOBJ_H_
#define _JRD_INTLOBJ_H_

#define	IB_LANGDRV_VERSION	1

typedef USHORT	(*FPTR_SHORT)();
typedef SSHORT	(*FPTR_short)();

typedef	USHORT		CHARSET_ID;         /* Valid range is 0 .. 255 */
typedef	USHORT		COLLATE_ID;         /* Valid range is 0 .. 254 */
typedef	USHORT		TTYPE_ID;           /* High byte is COLLATE_ID, low is CHARSET_ID */
typedef SCHAR		ASCII;

typedef unsigned char   ISC_NCHAR;              /* Narrow Char */
typedef unsigned short  ISC_WCHAR;              /* Wide   Char */
typedef unsigned char   ISC_MBCHAR;             /* Multibyte Char */

typedef struct texttype {
    struct blk	texttype_blk;
    USHORT	texttype_version;		/* version ID of object */
    USHORT	texttype_flags;			/* miscellanous flags */
    TTYPE_ID	texttype_type;			/* Interpretation ID */
    ASCII	*texttype_name;
    CHARSET_ID	texttype_character_set;		/* ID of base character set */
    SSHORT	texttype_country;		/* ID of base country values */
    BYTE	texttype_bytes_per_char;	/* max bytes per character */
    ULONG	texttype_license_mask;		/* required bits for license */

    /* MUST BE ALIGNED */
    FPTR_SHORT	texttype_fn_init;
    FPTR_SHORT	texttype_fn_key_length;
    FPTR_SHORT	texttype_fn_string_to_key;
    FPTR_short	texttype_fn_compare;
    FPTR_SHORT	texttype_fn_to_upper;		/* convert one ch to uppercase */
    FPTR_SHORT	texttype_fn_to_lower;		/* One ch to lowercase */
    FPTR_short	texttype_fn_str_to_upper;	/* Convert string to uppercase */
    FPTR_SHORT	texttype_fn_to_wc;		/* convert string to wc */
    FPTR_SHORT	texttype_fn_contains;		/* s1 contains s2? */
    FPTR_SHORT	texttype_fn_like;		/* s1 like s2? */
    FPTR_SHORT	texttype_fn_matches;		/* s1 matches s2 */
    FPTR_SHORT	texttype_fn_sleuth_check;	/* s1 sleuth s2 */
    FPTR_SHORT	texttype_fn_sleuth_merge;	/* aux function for sleuth */
    FPTR_short	texttype_fn_mbtowc;		/* get next character */
    BYTE	*texttype_collation_table;
    BYTE	*texttype_toupper_table;
    BYTE	*texttype_tolower_table;
    BYTE	*texttype_expand_table;
    BYTE	*texttype_compress_table;
    BYTE	*texttype_misc;			/* Used by some drivers */
    ULONG	*texttype_unused [4];		/* spare space */
} *TEXTTYPE;

#define TEXTTYPE_init               1	/* object has been init'ed */
#define TEXTTYPE_reverse_secondary  2	/* Reverse order of secondary keys */
#define TEXTTYPE_ignore_specials    4	/* Do not put special values in keys */
#define TEXTTYPE_expand_before      8	/* Expansion weights before litagure */



typedef struct csconvert {
        struct blk 	csconvert_blk;
	USHORT		csconvert_version;
	USHORT		csconvert_flags;
	SSHORT		csconvert_id;
	ASCII		*csconvert_name;
	CHARSET_ID	csconvert_from;
	CHARSET_ID	csconvert_to;
	FPTR_SHORT	csconvert_convert;
	BYTE		*csconvert_datatable;
	BYTE		*csconvert_misc;
	ULONG		*csconvert_unused [2];
} *CSCONVERT;

/* values for csconvert_flags */

#define CONVERTTYPE_init	1	/* object has been init'ed */

/* Conversion error codes */

#define	CS_TRUNCATION_ERROR	1	/* output buffer too small	*/
#define	CS_CONVERT_ERROR	2	/* can't remap a character      */
#define	CS_BAD_INPUT		3	/* input string detected as bad */

#define	CS_CANT_MAP		0	/* Flag table entries that don't map */




typedef struct charset {
        struct blk		charset_blk;
	USHORT			charset_version;
	USHORT			charset_flags;
	CHARSET_ID		charset_id;
	ASCII			*charset_name;
	BYTE			charset_min_bytes_per_char;
	BYTE			charset_max_bytes_per_char;
	BYTE			charset_space_length;
	BYTE			*charset_space_character;

	/* Must be aligned */
	FPTR_SHORT		charset_well_formed;
	struct	csconvert	charset_to_unicode;
	struct	csconvert	charset_from_unicode;

	VEC			charset_converters;
	VEC			charset_collations;
	ULONG			*charset_unused [2];
} *CHARSET;

/* values for charset_flags */

#define	CHARSET_init	1
#define CHARSET_narrow	2
#define CHARSET_multi	4
#define CHARSET_wide	8

#endif /* _JRD_INTLOBJ_H_ */
