
#pragma package(smart_init)

/*
 *  LC_Example.c
 *
 *  This file is an example texttype implementation.
 *  In this example, the collation implemented is a "Book Title" collation.
 *  In book titles, a leading simple word is ignored.  Example simple
 *  words are "The", "a", "An".
 *  So "The Song of the Rose" sorts as if it were written
 *  "Song of the Rose".
 *
 */

#include "cs_example.h"

/* These are all the functions called by InterBase via pointers in
 * the texttype object.
 */
extern USHORT	example_key_length();
extern USHORT	example_string_to_key();
extern SSHORT	example_compare();
extern USHORT	example_to_upper();
extern SSHORT	example_str_to_upper();
extern USHORT	example_to_lower();
extern SSHORT	example_get_wchar();



/*
 *  Class initialization function.
 *
 *  This function is given a zero-initialized TEXTTYPE data structure
 *  for initialization of all interface functions.
 *  If a function is not implemented within the class, do not touch
 *  the relevent function pointer -- it will be post-initialized by
 *  the engine after this function is called.
 *
 *  The function will be called once per database, when the database
 *  refers to the texttype being defined.  Note: this is not once per
 *  server load, but every time a database is loaded by the server.
 *
 * RETURN:
 *      0           if no errors
 *      1           if any error.
 *                  The engine will treat any error as fatal.
 */
unsigned short LC_win1252_example(
    TEXTTYPE cache,
    SSHORT  parm1,
    SSHORT  dummy)
{
/* Internal name for the texttype class */
static CONST ASCII	POSIX[] =       "EXAMPLE.WIN1252";

cache->texttype_name =			    (SCHAR *) POSIX;
cache->texttype_version =		    IB_LANGDRV_VERSION;
cache->texttype_type =			    parm1;
cache->texttype_character_set =		CS_WIN1252;
cache->texttype_country =		    0;                  /* Not used */
cache->texttype_bytes_per_char =	1;
cache->texttype_fn_init =		    LC_win1252_example;
cache->texttype_fn_key_length =		example_key_length;
cache->texttype_fn_string_to_key =	example_string_to_key;
cache->texttype_fn_compare =		example_compare;
cache->texttype_fn_to_upper =		example_to_upper;
cache->texttype_fn_to_lower =		example_to_lower;
cache->texttype_fn_str_to_upper =	example_str_to_upper;
cache->texttype_fn_mbtowc =		    example_get_wchar;

/* The following pointers are used to store pointers to any datatables
 * the texttype needs.   The engine does not actually access any of these
 * tables directly -- so they can be used for any purpose */

cache->texttype_collation_table =	(BYTE *) NULL;
cache->texttype_toupper_table =		(BYTE *) NULL;
cache->texttype_tolower_table =		(BYTE *) NULL;
cache->texttype_compress_table =	(BYTE *) NULL;
cache->texttype_expand_table =		(BYTE *) NULL;

return 0;
}



#define ASCII_SPACE	    32			            /* ASCII code for space */
#define ASCII_UPPER_A	65			            /* ASCII code for 'A' */
#define ASCII_LOWER_A	(ASCII_UPPER_A + 32)	/* ASCII code for 'a' */
#define ASCII_UPPER_Z	90			            /* ASCII code for 'Z' */
#define ASCII_LOWER_Z	(ASCII_UPPER_Z + 32)	/* ASCII code for 'z' */

#define	ASCII7_UPPER(ch) \
	((((UCHAR) (ch) >= (UCHAR) ASCII_LOWER_A) && ((UCHAR) (ch) <= (UCHAR) ASCII_LOWER_Z)) \
		? (UCHAR) ((ch)-ASCII_LOWER_A+ASCII_UPPER_A) \
		: (UCHAR) (ch))
#define	ASCII7_LOWER(ch) \
	((((UCHAR) (ch) >= (UCHAR) ASCII_UPPER_A) && ((UCHAR) (ch) <= (UCHAR) ASCII_UPPER_Z)) \
		? (UCHAR) ((ch)-ASCII_UPPER_A+ASCII_LOWER_A) \
		: (UCHAR) (ch))


/*
 * key_length (in_len)
 *
 * For an input string of (in_len) bytes, return the maximum
 * key buffer length.
 *
 * This is used for index buffer allocation within the engine.
 *
 * The InterBase engine constrains the return value to the range
 * (inLen .. MAX_KEY), where MAX_KEY is the limit on index keys.
 * (Currently approximately 254 in InterBase 6.0)
 *
 * This function must return the "expected worst case" key for ANY input
 * string of length inLen.
 *
 * Example:
 *  German has the  character <s-set>.   This character sorts as two lower
 *  case s characters.  However, when compared to a (true) ss, it sorts
 *  after (as a secondary ordering).
 * Input String:
 *     <s-set>         Key: <S> <S> <1> <1> <s> <s>
 *      ss              Key: <S> <S> <0> <0> <s> <s>
 *  In the "worst case", the sorting key is 6 times as long as the input
 *  string.  However, the "worst case" never occurs in actual usage
 *  (a string consisting entirely of  characters).
 *  In the "expected worst case" the sorting key is only 3 times as long
 *  as the input string for lexical ordering.
 *  However, a one character column is always a possibility, so for
 *  German, the following is an appropriate return.
 *      return MAX(6, 3*inLen);
 *
 * RETURNS
 *      Maximum length of the key for an input string of length (inLen)
 *
 */
USHORT	example_key_length (
    TEXTTYPE	obj,
    USHORT	    inLen)
{
return inLen;
}


/* example_simple_words
 *
 * This table lists the simple words that are ignored at the start of
 * an input string for the sake of this example collation.
 */

static CONST struct
    {
    UCHAR    text [5];
    USHORT   length;
    }
example_simple_words [] =
    {
        { "the ", 4},
        { "a ", 2},
        { "an ", 3},
        { "el ", 3}
    };

#define COUNT(x)    (sizeof(x)/sizeof((x)[0]))

/*
 * strncmpi()
 *
 * On some platforms, it is not possible to reach system functions
 * from the shared module.  In those cases, the international module must
 * locally define all functions needed.
 *
 * Performs case insenstive comparision of two strings, up to length len.
 * NOTE: this implementation ignores the possibility of a NULL byte
 * in either input string.
 *
 * RETURNS
 *  negative value      if s1 < s2
 *  0                   if s1 = s2
 *  positive value      if s1 > s2
 */
static short local_strncmpi (const char *s1, const char *s2, int len)
{
UCHAR   c1, c2;
while (len-- > 0)
    {
    c1 = ASCII7_UPPER (*((UCHAR *)s1));
    c2 = ASCII7_UPPER (*((UCHAR *)s2));
    if (c1 != c2)
        return ((short) (c1 - c2));
    s1++;
    s2++;
    }
return 0;
}


/*
 * match_simple_word
 *
 * If the input string starts with a simple word, that should be ignored
 * in our example collation, return the length, in bytes, of the word that
 * should be skipped.
 *
 * RETURNS
 *      0       pInChar does not start with a simple word.
 *     n > 0    pInChar starts with a simple word of length n
 */
static USHORT match_simple_word (USHORT iInLen, const UCHAR *pInChar)
{
int i;
for (i = 0; i < COUNT (example_simple_words); i++)
    if (example_simple_words[i].length < iInLen &&
        (local_strncmpi ((char *)example_simple_words[i].text, (char *)pInChar,
            example_simple_words[i].length) == 0))
        {
        return example_simple_words[i].length;
        };
return 0;
}



/*
 * string_to_key ()
 *
 *  Convert a user string to a sequence that will collate bytewise.
 *
 *  For ASCII type collation (codepoint collation) this mearly
 *  involves stripping the space character off the key.
 *
 * NOTE: SQL requires that trailing spaces on a string are insignificant.
 *       This function must return the same key for two strings that
 *       differ only by trailing spaces.  Note that the binary value of
 *       a "space character" is determined by the character set.
 *       Since this example collation is defined for only a single character
 *       set we can hardcode the value for an ASCII_SPACE.
 *
 * RETURN:
 *		Length, in bytes, of returned key
 */
USHORT	example_string_to_key (obj, iInLen, pInChar, iOutLen, pOutChar, partial)
    TEXTTYPE	obj;
    USHORT	iInLen;
    BYTE	*pInChar;
    USHORT	iOutLen;
    BYTE	*pOutChar;
    USHORT	partial;
{
BYTE	*outbuff;
BYTE	*inbuff;
USHORT  simple;

assert (pOutChar != NULL);
assert (pInChar  != NULL);
assert (iOutLen  >= example_key_length (obj, iInLen));

/* point inbuff at last character */
inbuff = pInChar + iInLen - 1;

/* skip backwards over all spaces & reset input length */
while ((inbuff >= pInChar) && (*inbuff == ASCII_SPACE))
    inbuff--;
iInLen = (USHORT) (inbuff - pInChar + 1);

/* Now make the key -- see if the input string starts with
   a simple word */

simple = match_simple_word (iInLen, pInChar);
if (simple)
    {
    /* We match a simple word -- skip it in the input stream */
    pInChar += simple;
    iInLen -= simple;
    };

/* Copy the rest of the input string to become the sorting key */

outbuff = pOutChar;
while (iInLen-- && iOutLen--)
    {
    *outbuff++ = *pInChar++;
    }
return (USHORT)(outbuff - pOutChar);
}

/*
 * all_spaces()
 *
 * Utility function to determine if the given input string consists
 * entirely of space characters.
 * This is important because SQL requires that trailing spaces on a string
 * are insignificant.
 *
 * RETURN
 *      TRUE        The string is all spaces.
 *      FALSE       The string is NOT all spaces.
 */
static SSHORT	all_spaces (
     BYTE	*s,
     USHORT	len)
{
assert (s != NULL);
while (len-- > 0)
    if (*s++ != ASCII_SPACE)
	    return FALSE;
return TRUE;
}

/* compare()
 *
 * Compare two strings (s1 of length l1) and (s2 of length l2).
 *
 * It is vital that compare of strings s1 & s2 returns the same
 * answer as string_to_key(s1) compared with string_to_key(s2).
 *
 * However, string_to_key() can assume a maximal length on the output
 * key (as index keys are limited to MAX_KEY bytes) -- whereas
 * compare() must handle strings of unlimited length.
 *
 * Generally, it is NOT acceptable to implement compare() by calling
 * string_to_key() on each string and comparing the resulting strings.
 * (as this method does not guarantee strings of unlimited length can
 * be compared).
 *
 * Return
 *      -1 if s1 < s2
 *      0  if s1 == s2
 *      1  if s1 > s2
 */
SSHORT	example_compare (obj, l1, s1, l2, s2)
     TEXTTYPE	obj;
     USHORT	l1;
     BYTE	*s1;
     USHORT	l2;
     BYTE	*s2;
{
USHORT	len;
USHORT	i;
USHORT  simple;

assert (obj != NULL);
assert (s1 != NULL);
assert (s2 != NULL);

/* Skip any simple word at the beginning of S1 or S2 */
simple = match_simple_word (l1, s1);
if (simple)
    {
    l1 -= simple;
    s1 += simple;
    };
simple = match_simple_word (l2, s2);
if (simple)
    {
    l2 -= simple;
    s2 += simple;
    }

/* Now compare the strings for equality for the length of the shorter */
len  = MIN (l1, l2);
for (i = 0; i < len; i++)
    {
    if (s1 [i] == s2 [i])
	    continue;
    else if (all_spaces (&s1 [i], (USHORT) (l1-i)))
	    return -1;
    else if (all_spaces (&s2 [i], (USHORT) (l2-i)))
	    return 1;
    else if (s1 [i] < s2 [i])
	    return -1;
    else
	    return 1;
    }

/* Strings are equal to the length of the shorter */

/* If S1 is the longer string, see if the remainder consists of spaces only */
if (l1 > len)
    {
    if (all_spaces (&s1 [len], (USHORT) (l1-len)))
	    return 0;
    return 1;
    }

/* If S2 is the longer string, see if the remainder consists of spaces only */
if (l2 > len)
    {
    if (all_spaces (&s2 [len], (USHORT) (l2-len)))
	    return 0;
    return -1;
    }

/* Strings were of equal length!  They are therefore equal */
return (0);
}


/*
 * to_upper()
 *
 * Input is a single character.
 * Output is that character converted to uppercase.
 *
 * In a multibyte character set, the input character is a WCHAR.
 *
 * RETURNS
 *      Character converted to uppercase (or unmodified if not a lowercase)
 */
USHORT	example_to_upper (obj, ch)
    TEXTTYPE	obj;
    BYTE	ch;
{
return ( (USHORT) ASCII7_UPPER (ch));
}


/*
 * str_to_upper
 *
 * Input string (pStr of iLen bytes) is converted to uppercase
 * and placed in output buffer (pOutStr of iOutLen bytes).
 *
 * Note that it is possible for str_to_upper to return a string of different
 * length than the input string.  For instance,
 *  German <s-set>  uppercases to SS (two characters).
 * It is not necessary that str_to_upper return the same result as
 * to_upper on each character.  (Though it is a really good idea!)
 *
 * RETURN
 *      -1 if any form of error.
 *      Otherwise, The length of data in the pOutStr buffer.
 */
SSHORT	example_str_to_upper (obj, iLen, pStr, iOutLen, pOutStr)
    TEXTTYPE	obj;
    USHORT	iLen;
    BYTE	*pStr;
    USHORT	iOutLen;
    BYTE	*pOutStr;
{
BYTE	*p;
assert (pStr != NULL);
assert (pOutStr != NULL);
assert (iOutLen >= iLen);
p = pOutStr;
while (iLen && iOutLen)
    {
    *pOutStr++ = ASCII7_UPPER (*pStr);
    pStr++;
    iLen--;
    iOutLen--;
    };
if (iLen != 0)
    return (-1);
return (SSHORT)(pOutStr - p);
}



/* to_lower()
 *
 *  Input: character ch
 *  Output: ch converted to lowercase.
 *
 *  This function is not currently used by the engine (as of InterBase 6.0)
 *  the SQL LOWER() function had not been implemented.
 *
 * RETURNS
 *     The character converted to lowercase.
 */
USHORT	example_to_lower (obj, ch)
    TEXTTYPE	obj;
    BYTE	ch;
{
return (ASCII7_LOWER (ch));
}


/*
 *  get_wchar()
 *
 *	Get the next character from the multibyte or narrow character
 *	input stream, return it as a wchar (unsigned short).
 *
 *  NOTE: The WC pointer can be NULL -- in this event the engine
 *  is scanning a string to determine the allocation length of
 *  the equivilant WCHAR string.
 *
 *  NOTE: get_wchar() must NOT change character sets.  This function does
 *  not perform conversion to Unicode -- it mearly changes the
 *  form-of-use of the current character to a WCHAR form of use.
 *  This function will primarily be used by the engine for multibyte
 *  character sets.
 *
 *  get_wchar() is really a method belonging to a character set, instead
 *  of a texttype.  However, it was misimplemented into texttype, so here
 *  it must be.
 *
 *  RETURNS:
 *	Count of bytes consumed from the input stream for the current WCHAR
 *  *WC (if WC != NULL) is filled in with the consumed WCHAR.
 *  -1 if no more input.
 *  Nothing if error.
 */

typedef unsigned short  WCHAR;
SSHORT example_get_wchar (obj, wc, ptr, count)
    TEXTTYPE	obj;
    WCHAR	*wc;
    UCHAR	*ptr;
    USHORT	count;
{
assert (obj);
assert (ptr);

if (count >= 1)
    {
    if (wc)
	    *wc = *ptr;
    return 1;
    }

/* No more input */
if (wc)
    *wc = 0;
return -1;		/* No more characters */
}


